@@ -517,6 +517,8 @@ sp_capture_reader_read_jitmap (SpCaptureReader *self)
       g_hash_table_insert (ret, GSIZE_TO_POINTER (addr), g_strdup (str));
     }
 
+  sp_capture_reader_bswap_jitmap (self, jitmap);
+
   self->pos += jitmap->frame.len;
 
   return g_steal_pointer (&ret);
@@ -654,7 +656,7 @@ sp_capture_reader_read_counter_set (SpCaptureReader *self)
         {
           guint j;
 
-          for (j = 0; j < G_N_ELEMENTS (set->values[0].values); i++)
+          for (j = 0; j < G_N_ELEMENTS (set->values[0].values); j++)
             {
               set->values[i].ids[j] = GUINT32_SWAP_LE_BE (set->values[i].ids[j]);
               set->values[i].values[j].v64 = GUINT64_SWAP_LE_BE (set->values[i].values[j].v64);
