@@ -35,16 +35,23 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
 #include <errno.h>
 #include <gio/gio.h>
 #include <gio/gunixfdlist.h>
-#include <polkit/polkit.h>
 #include <string.h>
 #include <sys/ioctl.h>
 #include <sys/mman.h>
 #include <sys/syscall.h>
 #include <unistd.h>
 
+#ifdef ENABLE_SYSPROFD
+# include <polkit/polkit.h>
+#endif
+
 #include "sp-perf-counter.h"
 
 #include "util.h"
@@ -118,7 +125,7 @@ G_DEFINE_BOXED_TYPE (SpPerfCounter,
                      (GBoxedCopyFunc)sp_perf_counter_ref,
                      (GBoxedFreeFunc)sp_perf_counter_unref)
 
-#if ENABLE_SYSPROFD
+#ifdef ENABLE_SYSPROFD
 static GDBusConnection *shared_conn;
 #endif
 
@@ -240,7 +247,7 @@ sp_perf_counter_flush (SpPerfCounter     *self,
        * which means there will always be space for one header, which means we
        * can safely dereference the size field.
        */
-      header = (struct perf_event_header *)(info->data + (tail & mask));
+      header = (struct perf_event_header *)(gpointer)(info->data + (tail & mask));
 
       if (header->size > head - tail)
         {
@@ -268,7 +275,7 @@ sp_perf_counter_flush (SpPerfCounter     *self,
           memcpy (b, info->data + (tail & mask), n_before);
           memcpy (b + n_before, info->data, n_after);
 
-          header = (struct perf_event_header *)b;
+          header = (struct perf_event_header *)(gpointer)b;
         }
 
       if (header->type == PERF_RECORD_SAMPLE)
@@ -496,6 +503,27 @@ get_authorized_proxy (void)
   return NULL;
 }
 
+static void
+sp_perf_counter_ping_cb (GObject      *object,
+                         GAsyncResult *result,
+                         gpointer      user_data)
+{
+  GDBusProxy *proxy = (GDBusProxy *)object;
+  g_autoptr(GTask) task = user_data;
+  g_autoptr(GVariant) ret = NULL;
+  GError *error = NULL;
+
+  g_assert (G_IS_DBUS_PROXY (proxy));
+  g_assert (G_IS_TASK (task));
+  g_assert (G_IS_ASYNC_RESULT (result));
+
+  ret = g_dbus_proxy_call_finish (proxy, result, &error);
+
+  if (error != NULL)
+    g_task_return_error (task, error);
+  else
+    g_task_return_boolean (task, TRUE);
+}
 
 static void
 sp_perf_counter_acquire_cb (GObject      *object,
@@ -504,6 +532,7 @@ sp_perf_counter_acquire_cb (GObject      *object,
 {
   g_autoptr(GTask) task = user_data;
   GPermission *permission = (GPermission *)object;
+  g_autoptr(GDBusProxy) proxy = NULL;
   GError *error = NULL;
 
   g_assert (G_IS_PERMISSION (permission));
@@ -516,7 +545,26 @@ sp_perf_counter_acquire_cb (GObject      *object,
       return;
     }
 
-  g_task_return_boolean (task, TRUE);
+  proxy = get_proxy ();
+
+  if (proxy == NULL)
+    {
+      /* We don't connect at startup, shouldn't happen */
+      g_task_return_new_error (task,
+                               G_IO_ERROR,
+                               G_IO_ERROR_FAILED,
+                               "Failed to create proxy");
+      return;
+    }
+
+  g_dbus_proxy_call (proxy,
+                     "org.freedesktop.DBus.Peer.Ping",
+                     NULL,
+                     G_DBUS_CALL_FLAGS_NONE,
+                     5000,
+                     g_task_get_cancellable (task),
+                     sp_perf_counter_ping_cb,
+                     g_object_ref (task));
 }
 
 static void
@@ -592,13 +640,13 @@ sp_perf_counter_authorize_async (GCancellable        *cancellable,
 
   task = g_task_new (NULL, cancellable, callback, user_data);
 
-#if ENABLE_SYSPROFD
+#ifdef ENABLE_SYSPROFD
   g_bus_get (G_BUS_TYPE_SYSTEM,
              cancellable,
              sp_perf_counter_get_bus_cb,
              g_object_ref (task));
 #else
-  g_task_return_new_error (error,
+  g_task_return_new_error (task,
                            G_IO_ERROR,
                            G_IO_ERROR_NOT_SUPPORTED,
                            "Sysprofd is not supported in current configuration");
@@ -650,7 +698,10 @@ sp_perf_counter_open (SpPerfCounter          *self,
 			"("
 				"["
 					"{'comm', <%b>},"
+#ifdef HAVE_PERF_CLOCKID
 					"{'clockid', <%i>},"
+					"{'use_clockid', <%b>},"
+#endif
 					"{'config', <%t>},"
 					"{'disabled', <%b>},"
 					"{'exclude_idle', <%b>},"
@@ -660,15 +711,17 @@ sp_perf_counter_open (SpPerfCounter          *self,
 					"{'sample_period', <%t>},"
 					"{'sample_type', <%t>},"
 					"{'task', <%b>},"
-					"{'type', <%u>},"
-					"{'use_clockid', <%b>}"
+					"{'type', <%u>}"
 				"],"
 				"%i,"
 				"%i,"
 				"%t"
 			")",
       (gboolean)!!attr->comm,
+#ifdef HAVE_PERF_CLOCKID
       (gint32)attr->clockid,
+      (gboolean)!!attr->use_clockid,
+#endif
       (guint64)attr->config,
       (gboolean)!!attr->disabled,
       (gboolean)!!attr->exclude_idle,
@@ -679,7 +732,6 @@ sp_perf_counter_open (SpPerfCounter          *self,
       (guint64)attr->sample_type,
       (gboolean)!!attr->task,
       (guint32)attr->type,
-      (gboolean)!!attr->use_clockid,
       (gint32)pid,
       (gint32)cpu,
       (guint64)flags);
