@@ -329,6 +329,7 @@ sp_window_set_state (SpWindow      *self,
       sp_window_set_profiler (self, profiler);
       break;
 
+    case SP_WINDOW_STATE_0:
     default:
       g_warning ("Unknown state: %0d", state);
       break;
@@ -379,18 +380,11 @@ sp_window_start_recording (SpWindow *self)
 {
   g_assert (SP_IS_WINDOW (self));
 
-  if (self->state == SP_WINDOW_STATE_RECORDING)
-    {
-      /* SpProfiler::stopped will move us to generating */
-      gtk_label_set_label (self->subtitle, _("Stopping…"));
-      sp_profiler_stop (self->profiler);
-      return;
-    }
-
   if ((self->state == SP_WINDOW_STATE_EMPTY) ||
       (self->state == SP_WINDOW_STATE_FAILED) ||
       (self->state == SP_WINDOW_STATE_BROWSING))
     {
+      gtk_revealer_set_reveal_child (self->info_bar_revealer, FALSE);
       sp_window_add_sources (self, self->profiler);
       sp_window_set_state (self, SP_WINDOW_STATE_RECORDING);
       sp_window_enable_stats (self);
@@ -407,7 +401,16 @@ sp_window_stop_recording (SpWindow *self)
   if (self->state == SP_WINDOW_STATE_RECORDING)
     {
       if (self->profiler != NULL)
-        sp_profiler_stop (self->profiler);
+        {
+          /* SpProfiler::stopped will move us to generating */
+          gtk_label_set_label (self->subtitle, _("Stopping…"));
+          /*
+           * In case that ::stopped takes a while to execute,
+           * disable record button immediately.
+           */
+          gtk_widget_set_sensitive (GTK_WIDGET (self->record_button), FALSE);
+          sp_profiler_stop (self->profiler);
+        }
     }
 }
 
@@ -762,7 +765,6 @@ sp_window_class_init (SpWindowClass *klass)
 static void
 sp_window_init (SpWindow *self)
 {
-  GAction *action;
   static GActionEntry action_entries[] = {
     { "close-capture", sp_window_close_capture },
     { "open-capture",  sp_window_open_capture },
